'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FieldStatus = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // @ts-check
/* eslint-disable no-param-reassign */
// This file contain any `private` method for the Veasy

exports.shouldChange = shouldChange;
exports.throwError = throwError;
exports.resetForm = resetForm;
exports.startValidating = startValidating;
exports.validate = validate;

var _is_js = require('is_js');

var _is_js2 = _interopRequireDefault(_is_js);

var _validationUtils = require('./validationUtils');

var _initializationUtils = require('./initializationUtils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var FieldStatus = exports.FieldStatus = {
  ok: 'ok',
  error: 'error',
  normal: 'normal'
};

/**
 * Check if we should change the state or not.
 *
 */
function shouldChange(oldState, newState) {
  var isErrorDifferent = oldState.status !== newState.status;
  var isValueDifferent = oldState.value !== newState.value;
  return isErrorDifferent || isValueDifferent;
}

/**
 * throw an error with defined text, usually calls by ruleRunner().
 */
function throwError(value, errorText) {
  // eslint-disable-next-line no-throw-literal
  throw { value: value, errorText: errorText, status: FieldStatus.error };
}

function resetForm(schema, state) {
  var newSchema = _extends({}, schema);
  delete newSchema.collectValues;
  var newState = _extends({}, state);
  var fieldNames = Object.keys(newSchema);
  fieldNames.forEach(function (name) {
    var initialFieldState = (0, _initializationUtils.createFieldState)(schema, name);
    newState[name].value = initialFieldState.value;
    newState[name].status = initialFieldState.status;
    newState[name].errorText = initialFieldState.errorText;
  });
  (0, _validationUtils.checkIsFormOK)(schema, newState);
  return newState;
}

function updateWhenNeeded(newFieldState, propName, update, schema) {
  var formState = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;

  var fieldState = _defineProperty({}, propName, newFieldState);
  if (formState === undefined) {
    update(fieldState);
    return;
  }

  var oldFieldState = formState[propName];
  var newFieldState1 = _extends({}, oldFieldState, fieldState[propName]);

  if (_is_js2.default.existy(oldFieldState) && _is_js2.default.existy(newFieldState)) {
    if (!shouldChange(oldFieldState, newFieldState)) return;
  } else {
    return;
  }

  var finalState = _extends({}, formState, _defineProperty({}, propName, newFieldState1));
  update((0, _validationUtils.checkIsFormOK)(schema, finalState));
}

function startValidating(target, schema, update, allState) {
  var targetName = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;

  var propName = targetName || target.name;

  if (_is_js2.default.not.existy(propName)) {
    // If the user includes non-form field component
    // Instead of throw, we should just ignore
    return undefined;
  }

  var fieldInfo = {
    value: target.value,
    schema: _defineProperty({}, propName, schema[propName])
  };

  return Promise.resolve(fieldInfo).then(function (info) {
    return (0, _validationUtils.rulesRunner)(info.value, info.schema, allState);
  }).catch(function (errorState) {
    return errorState;
  }).then(function (newFieldState) {
    return updateWhenNeeded(newFieldState, propName, update, schema, allState);
  });
}

function validate(e, schema, allState, update, targetName) {
  e.persist();
  startValidating(e.target, schema, update, allState, targetName);
}
//# sourceMappingURL=helpers.js.map