"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createInitialValue = createInitialValue;
exports.createNewFieldState = createNewFieldState;
exports.createFieldState = createFieldState;
exports.createInitialState = createInitialState;

var _is_js = require("is_js");

var _is_js2 = _interopRequireDefault(_is_js);

var _helpers = require("./helpers");

var _validationUtils = require("./validationUtils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Create initial value for a field if no default is provided.
 *
 */
function createInitialValue(schema) {
  if (_is_js2.default.propertyDefined(schema, 'default')) {
    return schema.default;
  } else if (_is_js2.default.propertyDefined(schema, 'min')) {
    return schema.min;
  }
  return '';
}

/**
 * Create a new state for a field in componentState.formStatus.fields.field
 *
 */
function createNewFieldState() {
  var needValue = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var fieldSchema = arguments[1];

  var result = {
    status: _helpers.FieldStatus.normal,
    errorText: ''
  };
  if (needValue) {
    result.value = createInitialValue(fieldSchema);
  }
  return result;
}

/**
 * When the schema contains a default rule
 */
function validateStateIfHasDefaultValue(schema, fieldName, fieldValue) {
  var result = void 0;
  if (_is_js2.default.propertyDefined(schema[fieldName], 'default')) {
    try {
      result = (0, _validationUtils.rulesRunner)(fieldValue, schema);
    } catch (err) {
      result = err;
    }
  }
  return result;
}

function createFieldState(schema, fieldName) {
  var initialFieldState = createNewFieldState(true, schema[fieldName]);

  var result = validateStateIfHasDefaultValue(schema, fieldName, initialFieldState.value);

  if (result) {
    return result;
  }
  return initialFieldState;
}

function createInitialState(schema, userState) {
  var initialState = _extends({}, userState);

  Object.keys(schema).forEach(function (fieldName) {
    if (fieldName === 'collectValues') return;
    initialState[fieldName] = createFieldState(schema, fieldName);
  });

  var schemaItems = Object.keys(schema);
  schemaItems.forEach(function (name) {
    if (_is_js2.default.propertyDefined(userState, name)) {
      initialState[name] = _extends({}, initialState[name], userState[name]);
    }
  });

  (0, _validationUtils.checkIsFormOK)(schema, initialState);

  return initialState;
}
//# sourceMappingURL=initializationUtils.js.map