"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.checkIsFormOK = checkIsFormOK;
exports.rulesRunner = rulesRunner;

var _is_js = require("is_js");

var _is_js2 = _interopRequireDefault(_is_js);

var _helpers = require("./helpers");

var _matchers = require("../ruleHandlers/matchers");

var _matchers2 = _interopRequireDefault(_matchers);

var _initializationUtils = require("./initializationUtils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } /* eslint-disable no-param-reassign */


/**
 * If all fields in the state has their status !== error
 * Then we will set the isFormOK to true then return the state.
 * Just mutate the value since it's already a new state object
 *
 */
function checkIsFormOK(schema, componentState) {
  var properties = Object.keys(schema);
  var isError = false;
  properties.some(function (prop) {
    if (prop === 'collectValues') return false;

    if (_is_js2.default.propertyDefined(schema[prop], 'isRequired') && schema[prop].isRequired === false && componentState[prop].status !== _helpers.FieldStatus.error) return false;

    if (componentState[prop].status === _helpers.FieldStatus.error) {
      isError = true;
      return true;
    }

    if (componentState[prop].status === _helpers.FieldStatus.normal) {
      if (_is_js2.default.not.propertyDefined(schema[prop], 'default')) {
        isError = true;
        return true;
      }

      if (schema[prop].default !== componentState[prop].value) {
        isError = true;
        return true;
      }
    }
    return false;
  });
  componentState.isFormOK = !isError;

  return componentState;
}

function handleBeforeValidation(fieldValue, handler) {
  if (_is_js2.default.function(handler)) {
    return handler(fieldValue);
  }
  /* eslint no-console: 0 */
  console.warn("[Veasy]: Expect beforeValidation to be a function while the value is " + handler);
  return fieldValue;
}

function extractUserDefinedMsg(handlerName, schema) {
  var result = { schema: schema, userErrorText: '' };

  // No user message, just return
  if (_is_js2.default.not.array(schema[handlerName])) return result;

  var currentSchema = schema[handlerName];

  // Handle the case where the value of rule is array
  if (_matchers.RuleWhichNeedsArray.includes(handlerName)) {
    // No user message, just return
    if (_is_js2.default.not.array(currentSchema[0])) return result;
  }

  // The most common case: [0] is rule and [1] is errText
  result.schema = _defineProperty({}, handlerName, currentSchema[0]);
  // eslint-disable-next-line prefer-destructuring
  result.userErrorText = currentSchema[1];
  return result;
}

function ruleRunner(ruleName, ruleHandler, fieldName, value, pschema) {
  var _extractUserDefinedMs = extractUserDefinedMsg(ruleName, pschema),
      schema = _extractUserDefinedMs.schema,
      userErrorText = _extractUserDefinedMs.userErrorText;

  if (_matchers.RuleWhichNeedsBoolean.includes(ruleName)) {
    if (schema[ruleName] === false) return;
  }

  var result = ruleHandler(fieldName, value, schema);
  if (result.isValid) return;

  (0, _helpers.throwError)(value, userErrorText || result.errorText);
}

function handleReliesOn(fieldReliesOnSchema, fieldState, allState) {
  var originalFieldState = _extends({}, fieldState);
  Object.keys(fieldReliesOnSchema).forEach(function (reliedField) {
    var reliesKeySchema = fieldReliesOnSchema[reliedField];
    Object.keys(reliesKeySchema).forEach(function (rule) {

      if (_is_js2.default.not.propertyDefined(_matchers2.default, rule)) return;

      try {
        ruleRunner(rule, _matchers2.default[rule], reliedField, allState[reliedField].value, // Here we need to swap the field value to the target value
        reliesKeySchema);
      } catch (err) {
        // Restore the original value
        err.value = originalFieldState.value;
        throw err;
      }
    });
  });
}

/**
 * It will run through the user's settings for a field,
 * and try matching to the matchers.js,
 * if according rule could be found,
 * it will then execute the according rule function.
 * For instance:
 * if user sets a `minLength` for a field,
 * This function will invoke the minLength()
 *
 */
function runMatchers(matcher, fieldState, fieldSchema, allState) {
  var fieldName = Object.keys(fieldSchema)[0];
  var schema = fieldSchema[fieldName];
  Object.keys(schema).forEach(function (ruleInSchema) {
    if (_is_js2.default.propertyDefined(matcher, ruleInSchema)) {
      // eslint-disable-next-line no-use-before-define
      ruleRunner(ruleInSchema, matcher[ruleInSchema], fieldName, fieldState.value, schema);
    } else if (ruleInSchema === 'beforeValidation') {
      fieldState.value = handleBeforeValidation(fieldState.value, schema.beforeValidation);
    } else if (ruleInSchema === 'reliesOn') {
      var fieldReliesOnSchema = fieldSchema[fieldName].reliesOn;
      handleReliesOn(fieldReliesOnSchema, fieldState, allState);
    }
    // TODO: Do something when the rule is not match
    // else if (ruleInSchema !== 'default') {
    // }
  });
  return fieldState;
}

/**
 * This is the main entry for all validator.
 * It will generate the initial state to start with
 *
 */
function rulesRunner(value, schema, allState) {
  var fieldState = (0, _initializationUtils.createNewFieldState)();
  fieldState.value = value;

  if (_is_js2.default.existy(value) && _is_js2.default.not.empty(value)) {
    fieldState.status = _helpers.FieldStatus.ok;
  }

  return runMatchers(_matchers2.default, fieldState, schema, allState);
}
//# sourceMappingURL=validationUtils.js.map