'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isNonEmptyObject = isNonEmptyObject;
exports.getConstructorErrorMessage = getConstructorErrorMessage;
exports.typeCheck = typeCheck;

var _is_js = require('is_js');

var _is_js2 = _interopRequireDefault(_is_js);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Check if an object is a non-empty object
 */
function isNonEmptyObject(obj) {
  return _is_js2.default.object(obj) && _is_js2.default.not.empty(obj);
}

/**
 * Return error message for checking the parameters of the constructor.
 */
function getConstructorErrorMessage(paramName, value) {
  return '[Veasy - ' + paramName + '] Expect: non empty object. Actual: ' + value;
}

/**
 * Type check for 2 parameters of the constructor
 *
 */
function typeCheck(component, schema) {
  var isComponentValid = isNonEmptyObject(component);
  var isSchemaValid = isNonEmptyObject(schema);

  if (!isComponentValid) {
    throw new Error(getConstructorErrorMessage('Parameter component', component));
  }

  if (!isSchemaValid) {
    throw new Error(getConstructorErrorMessage('Parameter schema', schema));
  }

  Object.keys(schema).forEach(function (prop) {
    if (!isNonEmptyObject(schema[prop])) {
      throw new Error(getConstructorErrorMessage('schema.' + prop, schema[prop]));
    }
  });
}
//# sourceMappingURL=veasyClassUtils.js.map